#include <stdio.h>
#include <stdlib.h>
#include <stddef.h>
#include <string.h>


#include <unistd.h>
#include <dirent.h>
#include <sys/stat.h>


#include <ysclass.h>
#include <ysbitmap.h>
#include <fsgui.h>
#include <fssimplewindow.h>


////////////////////////////////////////////////////////////
// Getting file list >>
static YSRESULT FsFindFileDirList(YsArray <YsString> &filelist,YsArray <YsString> &dirlist,const char dir[],const char ext[])
{
	int i;
	long hdl;
	unsigned attrib;
	YsString fn,tst,fulDir,uPrefix,uExt;
	DIR *dp;
	struct dirent *dr;


	uExt.Set(ext);
	uExt.Uncapitalize();

	if((dp=opendir(dir))==NULL)
	{
		return YSERR;
	}

	filelist.Set(0,NULL);

	hdl=-1;
	while((dr=readdir(dp))!=NULL)
	{
		if(dr->d_ino!=0 && dr->d_name[0]!='.')
		{
			struct stat st;

			fn.Set(dir);
			fn.Append("/");
			fn.Append(dr->d_name);

			stat(fn,&st);
			if((st.st_mode&S_IFMT)==S_IFDIR)
			{
				fn.Set(dr->d_name);
				dirlist.Append(fn);
			}
			else
			{
				fn.Set(dr->d_name);

				tst.Set(dr->d_name);
				tst.Uncapitalize();
				if(strncmp(tst,uPrefix,uPrefix.Strlen())==0 &&
				   strcmp(tst.GetArray()+tst.Strlen()-uExt.Strlen(),uExt)==0)
				{
					filelist.Append(fn);
				}
			}
		}
	}

	return YSOK;
}
// Getting file list <<
////////////////////////////////////////////////////////////



////////////////////////////////////////////////////////////
// File Dialog >>

class FsGuiFileDialog : public FsGuiDialog
{
public:
	FsGuiStatic *pathTxt;
	FsGuiListBox *lbx;
	FsGuiTextBox *txt;
	FsGuiButton *openBtn;
	FsGuiButton *cancelBtn;

#ifdef WIN32
	FsGuiListBox *drvLbx;
	FsGuiButton *moveDrvBtn;
#endif

	YSBOOL forOpenFile;

	YSBOOL selectionHasBeenMade;
	YSRESULT res;
	YsString ful;
	YsString pth;
	YsString fn;

	YsString curPath,curName,curExt;

	FsGuiFileDialog();
	void Initialize(void);

	YSRESULT MakeDialog(int x0,int y0,const char title[],YSBOOL forOpenFile);
	YSRESULT MakeFileList(const char iniPath[],const char iniName[],const char ext[]);
	void MoveDirectory(const char dir[]);
	YSBOOL MoveToSelectedDirectory(void);
#ifdef WIN32
	YSBOOL MoveToSelectedDrive(void);
#endif
	void Open(void);


	virtual void OnButtonClick(FsGuiButton *btn);
	virtual void OnListBoxSelChange(FsGuiListBox *lbx,int prevSel);
	virtual void OnListBoxDoubleClick(FsGuiListBox *lbx);



	YSRESULT SelectFile(
	    YsString &ful,YsString &pth,YsString &fn,
	    FsGuiCanvas *theApp,
	    const char iniPath[],const char iniName[],const char ext[],YSBOOL forOpenFile);

};

FsGuiFileDialog::FsGuiFileDialog()
{
	Initialize();
}

void FsGuiFileDialog::Initialize(void)
{
	FsGuiDialog::Initialize();

	lbx=NULL;
	txt=NULL;
	openBtn=NULL;
	cancelBtn=NULL;

	forOpenFile=YSFALSE;
	selectionHasBeenMade=YSFALSE;
	res=YSERR;
	ful.Set("");
	pth.Set("");
	fn.Set("");
}


YSRESULT FsGuiFileDialog::MakeDialog(int x0,int y0,const char title[],YSBOOL forOpenFile)
{
	Initialize();

	SetTopLeftCorner(x0,y0);
	SetTextMessage(title);

	SetSize(800,400);

	pathTxt=AddStaticText(1,FSKEY_NULL,"",32,3,YSTRUE);
	lbx=AddListBox(2,FSKEY_NULL,"",0,(const char **)NULL,12,32,YSTRUE);
	if(forOpenFile!=YSTRUE)
	{
		txt=AddTextBox(3,FSKEY_NULL,"","",32,YSTRUE);
	}
	openBtn=AddTextButton(4,FSKEY_ENTER,FSGUI_PUSHBUTTON,"Open",YSTRUE);
	cancelBtn=AddTextButton(5,FSKEY_ESC,FSGUI_PUSHBUTTON,"Cancel",YSTRUE);

#ifdef WIN32
	YsArray <YsString> drvLetter;
	YsArray <const char *> drvLetterPtr;
	int i;
	drvLetter.Set('Z'-'A'+1,NULL);
	drvLetterPtr.Set('Z'-'A'+1,NULL);
	for(i='A'; i<='Z'; i++)
	{
		drvLetter[i-'A'].Append((char)i);
		drvLetter[i-'A'].Append(":");
		drvLetterPtr[i-'A']=drvLetter[i-'A'];
	}

	drvLbx=AddListBox(6,FSKEY_NULL,"Drive",drvLetter.GetN(),drvLetterPtr,4,10,YSTRUE);
	moveDrvBtn=AddTextButton(7,FSKEY_NULL,FSGUI_PUSHBUTTON,"Change Drive",YSTRUE);
#endif

	SetTransparency(YSFALSE);

	Fit();

	return YSOK;
}

YSRESULT FsGuiFileDialog::MakeFileList(const char iniPath[],const char iniName[],const char ext[])
{
	YsArray <YsString> fileList,dirList;

	if(lbx!=NULL && FsFindFileDirList(fileList,dirList,iniPath,ext)==YSOK)
	{
		int i,id;

		lbx->ClearChoice();

		id=lbx->AddString("..",YSFALSE);
		lbx->SetDirectoryFlag(id,YSTRUE);

		for(i=0; i<dirList.GetN(); i++)
		{
			if(strcmp(dirList[i],".")!=0)
			{
				id=lbx->AddString(dirList[i],YSFALSE);
				lbx->SetDirectoryFlag(id,YSTRUE);
			}
		}

		for(i=0; i<fileList.GetN(); i++)
		{
			id=lbx->AddString(fileList[i],YSFALSE);
			lbx->SetDirectoryFlag(id,YSFALSE);
		}

		curPath.Set(iniPath);
		curName.Set(iniName);
		curExt.Set(ext);

		if(txt!=NULL)
		{
			txt->SetText(iniName);
		}

		if(pathTxt!=NULL)
		{
			pathTxt->SetText(curPath);
		}

		return YSOK;
	}
	return YSERR;
}

void FsGuiFileDialog::MoveDirectory(const char dir[])
{
	int i;

	if(strcmp(dir,"..")==0)
	{
		curPath.Set(this->curPath);
		for(i=curPath.Strlen(); i>=0; i--)
		{
			if(curPath[i]=='/' || curPath[i]=='\\')
			{
				curPath.Set(i,this->curPath);
				break;
			}
		}

		if(curPath.Strlen()==0)  // 2009/06/29
		{                        // 2009/06/29
			curPath.Set("/");    // 2009/06/29
		}                        // 2009/06/29

		MakeFileList(curPath,curName,curExt);
	}
	else if(strcmp(dir,".")==0)
	{
		// Do nothing.
	}
	else
	{
		curPath.Set(this->curPath);
#ifdef WIN32
		curPath.Append("\\");
#else
		curPath.Append("/");
#endif
		curPath.Append(dir);
		MakeFileList(curPath,curName,curExt);
	}
}

YSBOOL FsGuiFileDialog::MoveToSelectedDirectory(void)
{
	YsString txtA;
	if(NULL!=txt)
	{
		txt->GetText(txtA);
	}

	if(txtA[0]!=0)
	{
		YsString str;
		txtA.Capitalize();
		for(int i=0; YSOK==lbx->GetString(str,i); i++)
		{
			YsString txtB(str);
			txtB.Capitalize();

			if(strcmp(txtA,txtB)==0 && lbx->IsDirectory(i)==YSTRUE)
			{
				MoveDirectory(str);
				return YSTRUE;
			}
		}

		return YSFALSE;
	}

	const int sel=lbx->GetSelection();
	if(0<=sel && sel<lbx->GetNumChoice() && lbx->IsDirectory(sel)==YSTRUE)
	{
		YsString str;
		lbx->GetString(str,sel);
		MoveDirectory(str);
		return YSTRUE;
	}

	return YSFALSE;
}

#ifdef WIN32
YSBOOL FsGuiFileDialog::MoveToSelectedDrive(void)
{
	const char *drv;
	drv=drvLbx->GetSelectedString();
	if(drv!=NULL)
	{
		MakeFileList(drv,curName,curExt);
		return YSTRUE;
	}
	return YSFALSE;
}
#endif


void FsGuiFileDialog::Open(void)
{
	if(MoveToSelectedDirectory()!=YSTRUE)
	{
		pth=curPath;

		YsString txtContent;
		if(NULL!=txt)
		{
			txt->GetText(txtContent);
		}

		YsString selStr;
		if(NULL!=lbx)
		{
			lbx->GetSelectedString(selStr);
		}


		if(txtContent[0]!=0)
		{
			fn.Set(txtContent);
			selectionHasBeenMade=YSTRUE;
			CloseModalDialog(YSTRUE);
			res=YSOK;
		}
		else if(0!=selStr[0])
		{
			fn.Set(selStr);
			selectionHasBeenMade=YSTRUE;
			CloseModalDialog(YSTRUE);
			res=YSOK;
		}
		else
		{
			selectionHasBeenMade=YSFALSE;
		}

		if(selectionHasBeenMade==YSTRUE)
		{
			ful=pth;
#ifdef WIN32
			ful.Append("\\");
#else
			ful.Append("/");
#endif
			ful.Append(fn);
		}
	}
}

void FsGuiFileDialog::OnButtonClick(FsGuiButton *btn)
{
	if(btn==openBtn || lbx->CheckAndClearDoubleClick()==YSTRUE)
	{
		Open();
	}
	else if(btn==cancelBtn)
	{
		res=YSERR;
		selectionHasBeenMade=YSTRUE;  // This will terminate the modal loop.
		CloseModalDialog(YSTRUE);
	}
#ifdef WIN32
	else if(btn==moveDrvBtn || drvLbx->CheckAndClearDoubleClick()==YSTRUE)
	{
		MoveToSelectedDrive();
	}
#endif
}

void FsGuiFileDialog::OnListBoxSelChange(FsGuiListBox *lbx,int prevSel)
{
	YsString selStr;
	if(NULL!=txt && YSOK==lbx->GetSelectedString(selStr))
	{
		txt->SetText(selStr);
	}
}

void FsGuiFileDialog::OnListBoxDoubleClick(FsGuiListBox *lbx)
{
	Open();
}





YSRESULT FsGuiFileDialog::SelectFile(
    YsString &ful,YsString &pth,YsString &fn,
    FsGuiCanvas *theApp,
    const char iniPath[],const char iniName[],const char ext[],YSBOOL forOpenFile)
{
	const int x0=32,y0=32;
	YsString newFn;
	int ctr;

	Initialize();
	MakeDialog(x0,y0,"Select File",forOpenFile);
	if(YSOK!=MakeFileList(iniPath,iniName,ext))
	{
		const char *homeDir=getenv("HOME");
		if(NULL!=homeDir || YSOK!=MakeFileList(homeDir,iniName,ext))
		{
		  MakeFileList("/",iniName,ext);
		}
	}

	ful.Set("");
	pth.Set("");
	fn.Set("");
	newFn.Set("");

	theApp->AttachModalDialog(this);
	while(this==theApp->GetActiveModalDialog())
	{
		FsPollDevice();

		int key;
		while(FSKEY_NULL!=(key=FsInkey()))
		{
			theApp->KeyIn(key,(YSBOOL)FsGetKeyState(FSKEY_SHIFT),(YSBOOL)FsGetKeyState(FSKEY_CTRL),(YSBOOL)FsGetKeyState(FSKEY_ALT));
		}
		int chr;
		while(0!=(chr=FsInkeyChar()))
		{
			theApp->CharIn(chr);
		}

		int lb,mb,rb,mx,my;
		while(0!=FsGetMouseEvent(lb,mb,rb,mx,my))
		{
			theApp->SetMouseState((YSBOOL)lb,(YSBOOL)mb,(YSBOOL)rb,mx,my);
		}

		glClear(GL_COLOR_BUFFER_BIT|GL_DEPTH_BUFFER_BIT);
		theApp->Show();
		FsSwapBuffers();
		FsSleep(10);
	}

	if(res==YSOK)
	{
		ful=this->ful;
		fn=this->fn;
		pth=this->pth;
	}

	return res;
}

// File Dialog <<
////////////////////////////////////////////////////////////





static void BarToZero(char str[])
{
	int i,ln;
	ln=strlen(str);
	for(i=0; i<ln; i++)
	{
		if(str[i]=='|')
		{
			str[i]=0;
		}
	}
}

static void DecomposeExtList(char const *extLst[],int maxNumExt,char extbuf[])
{
	int numExt=0;
	int i=0;

	while(0!=extbuf[i] && numExt<maxNumExt-1)
	{
		if('|'!=extbuf[i])
		{
			extLst[numExt++]=extbuf+i;
			while(0!=extbuf[i])
			{
				if(extbuf[i]=='|')
				{
					break;
				}
				i++;
			}
			if(0==extbuf[i])
			{
				break;
			}
		}
		i++;
	}

	for(i=0; 0!=extbuf[i]; i++)
	{
		if(extbuf[i]=='|')
		{
			extbuf[i]=0;
		}
	}

	extLst[numExt]=NULL;
}

static void FormatExtension(char firstext[],char extformat[],char const *extlst[])
{
	if(extlst[0]==NULL || extlst==NULL)
	{
		strcpy(extformat,"(*.*)|*.*||");
		strcpy(firstext,"*.*");
		BarToZero(extformat);
	}
	else
	{
		int i,m;
		extformat[0]=0;

		for(m=0; m<2; m++)
		{
			for(i=0; extlst[i]!=NULL; i++)
			{
				char buf[256],*ext;
				strcpy(buf,extlst[i]);

				ext=buf;
				ext=(ext[0]=='*' ? ext+1 : ext);
				ext=(ext[0]=='.' ? ext+1 : ext);

				if(i!=0)
				{
					strcat(extformat,";");
				}
				strcat(extformat,"*.");
				strcat(extformat,ext);
			}
			strcat(extformat,"|");
		}

		for(i=0; extlst[i]!=NULL; i++)
		{
			char buf[256],*ext;
			strcpy(buf,extlst[i]);

			ext=buf;
			ext=(ext[0]=='*' ? ext+1 : ext);
			ext=(ext[0]=='.' ? ext+1 : ext);

			strcat(extformat,"*.");
			strcat(extformat,ext);
			strcat(extformat,"|*.");
			strcat(extformat,ext);
			strcat(extformat,"|");

			if(i==0)
			{
				sprintf(firstext,"*.%s",ext);
			}
		}
		strcat(extformat,"|");
		BarToZero(extformat);
	}
}

YSRESULT FsGuiOpenFileDialog(YsString &fn,const char title[],const char defFn[],const char defExt[],class FsGuiCanvas *theApp)
{
	FsGuiFileDialog fdlg;

	if(defExt!=NULL)
	{
		if(defExt[0]=='*')
		{
			defExt++;
		}
		if(defExt[0]=='.')
		{
			defExt++;
		}
	}
	else
	{
		defExt="*";
	}

	char iniPath[260],iniFile[260];
	if(defFn!=NULL && defFn[0]!=0)
	{
		YsSeparatePathFile(iniPath,iniFile,defFn);
		if(0==iniPath[0] || strcmp(iniPath,"./")==0 || 0==strcmp(iniPath,"."))
		  {
		    getcwd(iniPath,259);
		  }
	}
	else
	{
		getcwd(iniPath,259);//		strcpy(iniPath,".");
		strcpy(iniFile,"*.");
		strcat(iniFile,defExt);
	}

	YsString ful,pth,fil;
	if(fdlg.SelectFile(ful,pth,fil,theApp,iniPath,iniFile,defExt,YSTRUE)==YSOK)
	{
		ful.ReplaceExtension(defExt);
		fn=ful;
		return YSOK;
	}
	return YSERR;
}

YSRESULT FsGuiSaveFileDialog(YsString &fn,const char title[],const char defFn[],const char defExt[],class FsGuiCanvas *theApp)
{
	FsGuiFileDialog fdlg;

	if(defExt!=NULL)
	{
		if(defExt[0]=='*')
		{
			defExt++;
		}
		if(defExt[0]=='.')
		{
			defExt++;
		}
	}
	else
	{
		defExt="*";
	}

	char iniPath[260],iniFile[260];
	if(defFn!=NULL && defFn[0]!=0)
	{
		YsSeparatePathFile(iniPath,iniFile,defFn);
		if(0==iniPath[0] || strcmp(iniPath,"./")==0 || strcmp(iniPath,".")==0)
		  {
		    getcwd(iniPath,259);
		  }
	}
	else
	{
	  getcwd(iniPath,259); // strcpy(iniPath,".");
		strcpy(iniFile,"*.");
		strcat(iniFile,defExt);
	}

	YsString ful,pth,fil;
	if(fdlg.SelectFile(ful,pth,fil,theApp,iniPath,iniFile,defExt,YSFALSE)==YSOK)
	{
		ful.ReplaceExtension(defExt);
		fn=ful;
		return YSOK;
	}
	return YSERR;
}

