#include <stdio.h>
#include <fssimplewindow.h>
#include "fsguidialogutil.h"

////////////////////////////////////////////////////////////

void FsGuiMessageBoxDialog::Make(
    const wchar_t title[],const wchar_t msg[],const wchar_t yesBtn[],const wchar_t noBtn[],
    int nextActionYes,int nextActionNo)
{
	Initialize();

	this->nextActionCodeYes=nextActionYes;
	this->nextActionCodeNo=nextActionNo;

	SetTransparency(YSFALSE);

	SetTopLeftCorner(16,16);
	SetTextMessage(title);

	label=AddStaticText(0,FSKEY_NULL,msg,YSTRUE);
	label->SetFill(YSFALSE);
	label->SetDrawFrame(YSFALSE);

	this->yesBtn=AddTextButton(0,FSKEY_Y,FSGUI_PUSHBUTTON,yesBtn,YSTRUE);
	if(NULL!=noBtn)
	{
		this->noBtn=AddTextButton(1,FSKEY_N,FSGUI_PUSHBUTTON,noBtn,YSFALSE);
	}
	else
	{
		this->noBtn=NULL;
	}

	Fit();
}

void FsGuiMessageBoxDialog::OnKeyDown(int fsKey,YSBOOL shift,YSBOOL ctrl,YSBOOL alt)
{
	shift;
	ctrl;
	alt;

	if(NULL==noBtn)
	{
		if(FSKEY_ENTER==fsKey || FSKEY_ESC==fsKey || FSKEY_SPACE==fsKey)
		{
			CloseModalDialog(nextActionCodeYes);
		}
	}
	else
	{
		if(FSKEY_ESC==fsKey)  // Cannot tell the intention of SPACE or ENTER
		{
			CloseModalDialog(nextActionCodeNo);
		}
	}
}

void FsGuiMessageBoxDialog::OnButtonClick(FsGuiButton *btn)
{
	if(btn==yesBtn)
	{
		CloseModalDialog(nextActionCodeYes);
	}
	else if(NULL!=noBtn && btn==noBtn)
	{
		CloseModalDialog(nextActionCodeNo);
	}
}

////////////////////////////////////////////////////////////

template <const int N>
void FsGuiInputNumberDialogTemplate <N>::OnButtonClickTemplate(FsGuiButton *btn)
{
	if(btn==okBtn)
	{
		CloseModalDialog(returnCodeYes);
	}
	if(btn==cancelBtn)
	{
		CloseModalDialog(returnCodeNo);
	}
}



void FsGuiInputNumberDialog::Make(
    const double defNumber,const int belowDecimal,
    const wchar_t title[],const wchar_t msg[],const wchar_t label[],const wchar_t yesBtn[],const wchar_t noBtn[],
    int returnCodeYes,int returnCodeNo)
{
	Initialize();

	SetTextMessage(title);
	AddStaticText(0,FSKEY_NULL,msg,YSTRUE);
	AddStaticText(0,FSKEY_NULL,label,YSTRUE);
	numTxt[0]=AddTextBox(0,FSKEY_NULL,"","",8,YSFALSE);
	numTxt[0]->SetRealNumber(defNumber,belowDecimal);

	okBtn=AddTextButton(0,FSKEY_ENTER,FSGUI_PUSHBUTTON,yesBtn,YSTRUE);
	cancelBtn=AddTextButton(0,FSKEY_ESC,FSGUI_PUSHBUTTON,noBtn,YSFALSE);

	this->returnCodeYes=returnCodeYes;
	this->returnCodeNo=returnCodeNo;

	SetFocus(numTxt[0]);

	SetTransparency(YSFALSE);
	Fit();
}

const double FsGuiInputNumberDialog::GetNumber(void) const
{
	return numTxt[0]->GetRealNumber();
}

void FsGuiInputNumberDialog::OnButtonClick(FsGuiButton *btn)
{
	OnButtonClickTemplate(btn);
}



void FsGuiInputTwoNumberDialog::Make(
    const double dflt[2],const int belowDecimal,
    const wchar_t msg[],const wchar_t title[],const wchar_t label1[],const wchar_t label2[],
    const wchar_t yesBtn[],const wchar_t noBtn[],int returnCodeYes,int returnCodeNo)
{
	Initialize();

	SetTextMessage(title);
	AddStaticText(0,FSKEY_NULL,msg,YSTRUE);

	wchar_t const *label[]={label1,label2};

	for(int i=0; i<2; i++)
	{
		AddStaticText(0,FSKEY_NULL,label[i],YSTRUE);
		numTxt[i]=AddTextBox(0,FSKEY_NULL,"","",8,YSFALSE);
		numTxt[i]->SetRealNumber(dflt[i],belowDecimal);
	}

	okBtn=AddTextButton(0,FSKEY_ENTER,FSGUI_PUSHBUTTON,yesBtn,YSTRUE);
	cancelBtn=AddTextButton(0,FSKEY_ESC,FSGUI_PUSHBUTTON,noBtn,YSFALSE);

	this->returnCodeYes=returnCodeYes;
	this->returnCodeNo=returnCodeNo;

	SetFocus(numTxt[0]);

	SetTransparency(YSFALSE);

	Fit();

}

const double FsGuiInputTwoNumberDialog::GetNumber(int idx) const
{
	if(0==idx || 1==idx)
	{
		return numTxt[idx]->GetRealNumber();
	}
	return 0.0;
}

void FsGuiInputTwoNumberDialog::OnButtonClick(FsGuiButton *btn)
{
	OnButtonClickTemplate(btn);
}
