#ifndef YSBITMAP_IS_INCLUDED
#define YSBITMAP_IS_INCLUDED
/* { */

#include <stdio.h>

#ifndef YSRESULT_IS_DEFINED
#define YSRESULT_IS_DEFINED
typedef enum
{
	YSERR,
	YSOK
} YSRESULT;
#endif

#ifndef YSBOOL_IS_DEFINED
#define YSBOOL_IS_DEFINED
/*! Enum for boolearn. */
typedef enum
{
	YSFALSE,     /*!< False */
	YSTRUE,      /*!< True */
	YSTFUNKNOWN  /*!< Unable to tell true or false. */
} YSBOOL;
#endif

class YsBitmap
{
public:
	YsBitmap();
	~YsBitmap();
	void CleanUp(void);
	const YsBitmap &operator=(const YsBitmap &from);

	YSRESULT LoadBmp(const char fn[]);
	/*! fp must be opened with "rb". */
	YSRESULT LoadBmp(FILE *fp);
	YSRESULT LoadPng(const char fn[]);
	/*! fp must be opened with "rb". */
	YSRESULT LoadPng(FILE *fp);
private:
	YSRESULT MoveFromYsPngDecoder(class YsRawPngDecoder &pngDecoder);

public:
	YSRESULT PrepareBitmap(int wid,int hei);
	void Clear(unsigned char r,unsigned char g,unsigned char b,unsigned char a);
	YSRESULT ScaleCopy(int wid,int hei,const YsBitmap &from);
	YSRESULT Copy(const YsBitmap &from,int x0,int y0);

	/*! Copy from the bitmap incomingBmp starting from (inX0,inY0 to this bitmap from (toX0,toY0) 
	    for the wid pixels horizontally and hei pixels vertically. */
	YSRESULT Copy(const YsBitmap &incomingBmp,int inX0,int inY0,int toX0,int toY0,int wid,int hei);
	void Invert(void);

	void SetDirect(int wid,int hei,unsigned char *rgba);  // Buffer rgba will be managed by this object.

	YSRESULT InsecureTransfer(const YsBitmap &from,int x0,int y0);  // Incoming bmp must fit in this bmp.
	YSRESULT Transfer(const YsBitmap &from,int x0,int y0,unsigned int flags);
	// Flags:
	//    1: Horizontal Mirror
	//    2: Vertical Mirror
	//    4: Transpose
	//  128: Use Alpha
	YSRESULT VerticalScroll(int dy,unsigned char r,unsigned char g,unsigned char b,unsigned char a);

	YSRESULT SetR(unsigned x,unsigned y,unsigned char r);
	YSRESULT SetG(unsigned x,unsigned y,unsigned char g);
	YSRESULT SetB(unsigned x,unsigned y,unsigned char b);
	YSRESULT SetA(unsigned x,unsigned y,unsigned char a);
	YSRESULT SetRGBA(unsigned x,unsigned y,unsigned char r,unsigned char g,unsigned char b,unsigned char a);

	unsigned GetWidth(void) const;
	unsigned GetHeight(void) const;
	const unsigned char *GetRGBABitmapPointer(void) const;
	const unsigned char *GetRGBAPixelPointer(int x,int y) const;
	unsigned char *GetEditableRGBABitmapPointer(void);
	unsigned char *GetEditableRGBAPixelPointer(int x,int y);

	YSRESULT SaveBmp(const char fn[]) const;

	YSBOOL IsInRange(int x,int y) const;
	YSBOOL IsXInRange(int x) const;
	YSBOOL IsYInRange(int y) const;

protected:
	unsigned nx,ny;
	unsigned char *dat;
	unsigned char **lineTop;

	unsigned GetIntelUInt(const unsigned char dat[]);
	unsigned GetIntelUShort(const unsigned char dat[]);
	unsigned char *GetTopOfLine(int y) const;
};


/* } */
#endif
