#if (defined(_WIN32) || defined(_WIN64)) && !defined(_CRT_SECURE_NO_WARNINGS)
	// This disables annoying warnings VC++ gives for use of C standard library.  Shut the mouth up.
	#define _CRT_SECURE_NO_WARNINGS
#endif

#include <stdio.h>
#include <stdarg.h>

#include "ystextstream.h"
#include "ysstring.h"


YsTextFileInputStream::YsTextFileInputStream(FILE *fp)
{
	this->fp=fp;
}

const char *YsTextFileInputStream::Gets(YsString &str)
{
	return str.Fgets(fp);
}



YsTextFileOutputStream::YsTextFileOutputStream(FILE *fp)
{
	this->fp=fp;
}

YSRESULT YsTextFileOutputStream::Puts(const char str[])
{
	if(EOF!=fputs(str,fp))
	{
		return YSOK;
	}
	return YSERR;
}

YSRESULT YsTextFileOutputStream::Puts(const YsString &str)
{
	if(EOF!=fputs(str,fp))
	{
		return YSOK;
	}
	return YSERR;
}

YSRESULT YsTextFileOutputStream::Printf(const char fmt[],...)
{
	va_list arg_ptr;
	va_start(arg_ptr,fmt);
	vfprintf(fp,fmt,arg_ptr);
	va_end(arg_ptr);
	return YSOK;

}



YsListItem <YsString> *YsTextMemoryOutputStream::GetCurrentLine(void)
{
	if(NULL==curLine)
	{
		NewLine();
	}
	return curLine;
}

void YsTextMemoryOutputStream::NewLine(void)
{
	curLine=strList.Create();
	curLine->dat.Set("");
}

YsTextMemoryOutputStream::YsTextMemoryOutputStream() : strList(strAlloc)
{
	curLine=NULL;
}

YSRESULT YsTextMemoryOutputStream::Puts(const char str[])
{
	int i,head=0,count=0;
	for(i=0; 0!=str[i]; i++)
	{
		if('\n'==str[i])
		{
			YsListItem <YsString> *ptr=GetCurrentLine();
			ptr->dat.Append(count,str+head);
			NewLine();
			count=0;
			head=i+1;
		}
		else
		{
			count++;
		}
	}

	if(0<count)
	{
		YsListItem <YsString> *ptr=GetCurrentLine();
		ptr->dat.Append(count,str+head);
	}

	return YSOK;
}

YSRESULT YsTextMemoryOutputStream::Puts(const YsString &str)
{
	return Puts(str.Txt());
}

YSRESULT YsTextMemoryOutputStream::Printf(const char fmt[],...)
{
	const int iniBufSize=256;
	YsArray <char,256> buf;
	buf.Set(iniBufSize,NULL);
	buf[0]=0;

	for(;;)
	{
		const size_t bufSize=buf.GetN();
		va_list arg_ptr;

		va_start(arg_ptr,fmt);
		const int n=vsnprintf(buf,bufSize-1,fmt,arg_ptr);
		va_end(arg_ptr);

		if(n<0)
		{
			// This compiler s**ks!  It's not C99 compliant!  (I'm talking about Visual C++ 2008)
			buf.Resize(bufSize*2);
		}
		else if(n<(int)bufSize)
		{
			break;
		}
		else
		{
			// Good compiler!  It is C99 Compliant.
			buf.Resize(n+8);
		}
	}

	return Puts(buf);
}

const unsigned int YsTextMemoryOutputStream::GetN(void) const
{
	return strList.GetN();
}

const unsigned int YsTextMemoryOutputStream::GetNumLine(void) const
{
	return strList.GetN();
}

YsTextMemoryOutputStreamStringHandle YsTextMemoryOutputStream::FindNextLine(YsTextMemoryOutputStreamStringHandle ptr) const
{
	YsListItem <YsString> *linePtr=(YsListItem <YsString> *)ptr;
	return (YsTextMemoryOutputStreamStringHandle)strList.FindNext(linePtr);
}

YsTextMemoryOutputStreamStringHandle YsTextMemoryOutputStream::FindPrevLine(YsTextMemoryOutputStreamStringHandle ptr) const
{
	YsListItem <YsString> *linePtr=(YsListItem <YsString> *)ptr;
	return (YsTextMemoryOutputStreamStringHandle)strList.FindPrev(linePtr);
}

const YsString *YsTextMemoryOutputStream::GetLine(YsTextMemoryOutputStreamStringHandle ptr) const
{
	if(NULL!=ptr)
	{
		YsListItem <YsString> *linePtr=(YsListItem <YsString> *)ptr;
		return &linePtr->dat;
	}
	return NULL;
}
