#ifndef YSTEXTSTREAM_IS_INCLUDED
#define YSTEXTSTREAM_IS_INCLUDED
/* { */

#include "ysstring.h"
#include "yslist2.h"

class YsTextStream
{
};

class YsTextInputStream : public YsTextStream
{
public:
	virtual const char *Gets(YsString &str)=0;
};

class YsTextOutputStream : public YsTextStream
{
public:
	virtual YSRESULT Puts(const char str[])=0;
	virtual YSRESULT Puts(const YsString &str)=0;
	virtual YSRESULT Printf(const char fmt[],...)=0;
};



class YsTextFileInputStream : public YsTextInputStream
{
private:
	FILE *fp;
public:
	YsTextFileInputStream(FILE *fp);
	const char *Gets(YsString &str);
};

class YsTextFileOutputStream : public YsTextOutputStream
{
private:
	FILE *fp;
public:
	YsTextFileOutputStream(FILE *fp);
	YSRESULT Puts(const char str[]);
	YSRESULT Puts(const YsString &str);
	YSRESULT Printf(const char fmt[],...);
};



class YsTextMemoryOutputStreamStringHandleObj;
typedef YsTextMemoryOutputStreamStringHandleObj * YsTextMemoryOutputStreamStringHandle;

class YsTextMemoryOutputStream : public YsTextOutputStream
{
private:
	YsListAllocator <YsString> strAlloc;
	YsListContainer <YsString> strList;
	YsListItem <YsString> *curLine;

	YsListItem <YsString> *GetCurrentLine(void);
	void NewLine(void);

public:
	YsTextMemoryOutputStream(void);
	YSRESULT Puts(const char str[]);
	YSRESULT Puts(const YsString &str);
	YSRESULT Printf(const char fmt[],...);

	const unsigned int GetN(void) const;
	const unsigned int GetNumLine(void) const;
	YsTextMemoryOutputStreamStringHandle FindNextLine(YsTextMemoryOutputStreamStringHandle ptr) const;
	YsTextMemoryOutputStreamStringHandle FindPrevLine(YsTextMemoryOutputStreamStringHandle ptr) const;
	const YsString *GetLine(YsTextMemoryOutputStreamStringHandle ptr) const;
};


/* } */
#endif
