#include <stdio.h>
#include "ysglslutil.h"

void YsGLSLCompileAndLinkVertexAndFragmentShader(
    GLuint programId,
    GLuint vertexShaderId,
    const int nVertexShaderProgLine,const char * const vertexShaderProg[],
    GLuint fragmentShaderId,
    const int nFragShaderProgLine,const char * const fragShaderProg[])
{
	const int errMsgLen=1024;
	char errMsg[1024];
	int compileSta=99999,infoLogLength=99999,acquiredErrMsgLen=99999;
	int linkSta=99999;

	glShaderSource(
	    vertexShaderId,
	    nVertexShaderProgLine,(const char **)vertexShaderProg,
	    NULL); // Last NULL assumes each line is C string

	glShaderSource(
	    fragmentShaderId,
	    nFragShaderProgLine,(const char **)fragShaderProg,
	    NULL); // Last NULL assumes each line is C string

	glCompileShader(vertexShaderId);

	glGetShaderiv(vertexShaderId,GL_COMPILE_STATUS,&compileSta);
	glGetShaderiv(vertexShaderId,GL_INFO_LOG_LENGTH,&infoLogLength);
	glGetShaderInfoLog(vertexShaderId,errMsgLen-1,&acquiredErrMsgLen,errMsg);
	printf("Compile Status %d Info Log Length %d Error Message Length %d\n",compileSta,infoLogLength,acquiredErrMsgLen);

	if(GL_TRUE!=compileSta)
	{
		int i;
		printf("Error Message: \n%s\n",errMsg);
		for(i=0; i<nVertexShaderProgLine; ++i)
		{
			printf("%3d: %s",i+1,vertexShaderProg[i]);
		}
		printf("\n");
	}


	glCompileShader(fragmentShaderId);

	glGetShaderiv(fragmentShaderId,GL_COMPILE_STATUS,&compileSta);
	glGetShaderiv(fragmentShaderId,GL_INFO_LOG_LENGTH,&infoLogLength);
	glGetShaderInfoLog(fragmentShaderId,errMsgLen-1,&acquiredErrMsgLen,errMsg);
	printf("Compile Status %d Info Log Length %d Error Message Length %d\n",compileSta,infoLogLength,acquiredErrMsgLen);

	if(GL_TRUE!=compileSta)
	{
		int i;
		printf("Error Message: \n%s\n",errMsg);
		for(i=0; i<nFragShaderProgLine; ++i)
		{
			printf("%3d: %s",i+1,fragShaderProg[i]);
		}
		printf("\n");
	}

	glAttachShader(programId,vertexShaderId);
	glAttachShader(programId,fragmentShaderId);
	glLinkProgram(programId);
	glGetProgramiv(programId,GL_LINK_STATUS,&linkSta);
	glGetProgramiv(programId,GL_INFO_LOG_LENGTH,&infoLogLength);
	glGetProgramInfoLog(programId,errMsgLen-1,&acquiredErrMsgLen,errMsg);
	printf("Link Status %d Info Log Length %d Error Message Length %d\n",compileSta,infoLogLength,acquiredErrMsgLen);

	if(GL_TRUE!=linkSta)
	{
		printf("Error Message: \n%s\n",errMsg);
	}
}

void YsGLSLLoadIdentityMatrix4(GLuint uniformMatPos)
{
	const GLfloat mat[16]=
	{
		1.0f,0.0f,0.0f,0.0f,
		0.0f,1.0f,0.0f,0.0f,
		0.0f,0.0f,1.0f,0.0f,
		0.0f,0.0f,0.0f,1.0f
	};
	glUniformMatrix4fv(uniformMatPos,1,GL_FALSE,mat);
}

